package com.persagy.server.syn.services.jms.rel

import com.alibaba.fastjson.JSONArray
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.models.jms.SynchronousMessageRel
import com.persagy.server.syn.Configure
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.service.SObjectService
import org.springframework.stereotype.Service

/**
 * 设备所在的业务空间
 */
@Service
class EquipinSpaceSync {

    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageRelService = SObjectService(SMybatisDao(SynchronousMessageRel::class.java))
    /** 项目所属的集团 */
    private val dictProjectService = SObjectService(SMybatisDao(DictProject::class.java))
    /**
     * 创建关系
     *
     * @param synchronousMessageRel     关系对象
     * @param graphCode                 图类型
     * @param relCode                   关系类型
     */
    fun equipInSpaceCreate(synchronousMessageRel: SynchronousMessageRel,graphCode: String,relCode: String,fromIds:ArrayList<String>?){
        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", synchronousMessageRel.projectId!!)).entity()
            if (dictProject!=null) {
                synchronousMessageRel.sign = 0
                val toJsonObjArray = if (!fromIds.isNullOrEmpty()){
                    /** 关系列表 */
                    Configure.toJsonObjFromList(graphCode, relCode, fromIds, synchronousMessageRel.toId!!)
                } else {
                    /** 关系列表 */
                    Configure.toJsonObjArray(graphCode, relCode, synchronousMessageRel.fromId!!, synchronousMessageRel.toId!!)
                }
                val createRel = EquipApi.createRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObjArray)
                if (!createRel ){
                    synchronousMessageRel.sign = 2
                }
            } else {
                synchronousMessageRel.sign = 2
            }
            /** 更改标记 */
            synchronousMessageRelService.update(synchronousMessageRel, arrayListOf("sign"))
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 删除关系
     *
     * @param synchronousMessageRel     关系对象
     */
    fun delRel(synchronousMessageRel: SynchronousMessageRel,graphCode: String,relCode: String){
        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", synchronousMessageRel.projectId!!)).entity()
            if (dictProject!=null) {
                synchronousMessageRel.sign = 0
                val toJsonObj = Configure.toJsonObj(graphCode, relCode, synchronousMessageRel.fromId!!, synchronousMessageRel.toId!!)
                val delRel = EquipApi.delRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObj)
                if (!delRel){
                    synchronousMessageRel.sign = 2
                }
            } else {
                synchronousMessageRel.sign = 2
            }
            /** 更改标记 */
            synchronousMessageRelService.update(synchronousMessageRel, arrayListOf("sign"))
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 累计叠加关系
     */
    fun addRelSuperposition(synchronousMessageRel: SynchronousMessageRel,toIds: ArrayList<String>,graphCode: String,relCode: String){

        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", synchronousMessageRel.projectId!!)).entity()
            if (dictProject!=null) {
                synchronousMessageRel.sign = 0

                val toJsonObj = Configure.toJsonObj(graphCode, relCode, synchronousMessageRel.fromId!!, null)
                val delRel = EquipApi.delRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObj)
                if (delRel) {
                    /** 关系列表 */
                    val toJsonObjArray =
                            Configure.toJsonObjList(graphCode, relCode, synchronousMessageRel.fromId!!, toIds)

                    val createRel = EquipApi.createRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObjArray)
                    if (!createRel ){
                        synchronousMessageRel.sign = 2
                    }
                } else {
                    synchronousMessageRel.sign = 2
                }
                /** 更改标记 */
                synchronousMessageRelService.update(synchronousMessageRel, arrayListOf("sign"))

            } else {
                synchronousMessageRel.sign = 2
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 主对象和从对象相反的替换函数
     */
    fun addRelSuperpositionFan(synchronousMessageRel: SynchronousMessageRel,fromIds: ArrayList<String>,graphCode: String,relCode: String){

        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", synchronousMessageRel.projectId!!)).entity()
            if (dictProject!=null) {
                synchronousMessageRel.sign = 0
                for (id in fromIds){
                    val toJsonObj = Configure.toJsonObj(graphCode, relCode, id, null)
                    val delRel = EquipApi.delRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObj)
                    if (!delRel) {
                        synchronousMessageRel.sign = 2
                        break
                    }
                }
                if (synchronousMessageRel.sign == 0){
                    /** 关系列表 */
                    val toJsonObjArray =
                            Configure.toJsonObjFromList(graphCode, relCode, fromIds, synchronousMessageRel.toId!!)

                    val createRel = EquipApi.createRel(dictProject.groupCode!!, synchronousMessageRel.projectId!!, toJsonObjArray)
                    if (!createRel ){
                        synchronousMessageRel.sign = 2
                    }
                }

                /** 更改标记 */
                synchronousMessageRelService.update(synchronousMessageRel, arrayListOf("sign"))

            } else {
                synchronousMessageRel.sign = 2
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }



}